/*
 * This file is part of ComputerCraft - http://www.computercraft.info
 * Copyright Daniel Ratcliffe, 2011-2022. Do not distribute without permission.
 * Send enquiries to dratcliffe@gmail.com
 */
package dan200.computercraft.shared.common;

import dan200.computercraft.annotations.ForgeOverride;
import net.minecraft.core.BlockPos;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.Connection;
import net.minecraft.network.protocol.game.ClientboundBlockEntityDataPacket;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.BlockHitResult;

public abstract class GenericTile extends BlockEntity {
    public GenericTile(BlockEntityType<? extends GenericTile> type, BlockPos pos, BlockState state) {
        super(type, pos, state);
    }

    public void destroy() {
    }

    public final void updateBlock() {
        setChanged();
        var pos = getBlockPos();
        var state = getBlockState();
        getLevel().sendBlockUpdated(pos, state, state, Block.UPDATE_ALL);
    }

    public InteractionResult onActivate(Player player, InteractionHand hand, BlockHitResult hit) {
        return InteractionResult.PASS;
    }

    public void onNeighbourChange(BlockPos neighbour) {
    }

    public void onNeighbourTileEntityChange(BlockPos neighbour) {
    }

    protected void blockTick() {
    }

    protected double getInteractRange(Player player) {
        return 8.0;
    }

    public boolean isUsable(Player player) {
        if (player == null || !player.isAlive() || getLevel().getBlockEntity(getBlockPos()) != this) return false;

        var range = getInteractRange(player);
        var pos = getBlockPos();
        return player.getCommandSenderWorld() == getLevel() &&
            player.distanceToSqr(pos.getX() + 0.5, pos.getY() + 0.5, pos.getZ() + 0.5) <= range * range;
    }

    @ForgeOverride // FIXME: Implement this: I'd forgotten about this
    public final void onDataPacket(Connection net, ClientboundBlockEntityDataPacket packet) {
        var tag = packet.getTag();
        if (tag != null) handleUpdateTag(tag);
    }

    @ForgeOverride
    public void handleUpdateTag(CompoundTag tag) {
    }
}
